#include "common.h"

#define AST_STS_ERROR 	"Response: Error"
#define AST_STS_OK		"Response: Success"
#define AST_EVT_COMP	"Event: PeerlistComplete"

typedef struct
{	
	int		nstatus ;
	int		nlistcomplete ;
} AST_AGI_T ;

int sub_agi_peers(int nsoc);
int sub_agi_event(int nsoc);
int sub_agi_login(int nsoc);
int sub_read_agi(int nsoc, AST_AGI_T *dst);

int main(int nargc, char *pargv[])
{
	CONFIG_INFO_T	ftpt ;
	int				nsoc ;

    // ===========================================================================
    //  Step0: Read Config File
    // ===========================================================================

	memset( &ftpt, 0, sizeof(CONFIG_INFO_T) ) ;

    if ( sub_read_args(&ftpt, nargc, pargv) != STS_OK )
    {
        exit(0);
    }

    if ( sub_read_config(&ftpt) != STS_OK )
    {
        exit(0);
    }

	// ===========================================================================
	//	Initialize
	// ===========================================================================

	sub_logname( ftpt.dir_logdir, "ssw_file_delete_%s.log" ) ;

	sub_logput ( LOG_NORML, "===== Asterisk Server Control (%s) Start ================", FVERSION ) ;
	sub_logput ( LOG_NORML, "Asterisk Host: %s", ftpt.ast_host ) ;
	sub_logput ( LOG_NORML, "Asterisk Port: %s", ftpt.ast_port ) ;

    signal( SIGABRT, sub_signal ) ;
    signal(  SIGFPE, sub_signal ) ;
    signal(  SIGILL, sub_signal ) ;
    signal(  SIGINT, sub_signal ) ;
    signal( SIGSEGV, sub_signal ) ;
    signal( SIGTERM, sub_signal ) ;
    signal( SIGUSR1, sub_signal ) ;
    signal( SIGUSR2, sub_signal ) ;
    signal( SIGHUP,  sub_signal ) ;
    signal( SIGCHLD, SIG_IGN ) ;

	// ===========================================================================
	//	Connect To Asterisk Server
	// ===========================================================================

	if ( (nsoc = sub_connect_asterisk(ftpt.ast_host, atoi(ftpt.ast_port))) < 0 )
	{
		sub_logput( LOG_ERROR, "Connect To asterisk (%s, %s) failed", ftpt.ast_host, ftpt.ast_port ) ;
		exit(0);
	}

	sub_logput( LOG_NORML, "connected to asterisk (%s,%s)", ftpt.ast_host, ftpt.ast_port ) ;

	if ( sub_agi_login(nsoc) != STS_OK )
	{
		sub_terminate_sock( nsoc ) ;
		exit(0);
	}

	sub_agi_event(nsoc);

	sub_agi_peers(nsoc);

	// ===========================================================================
	//	Close Socket
	// ===========================================================================

	sub_terminate_sock( nsoc ) ;

	return STS_OK ;
}

int sub_agi_login(int nsoc)
{
	char		sndbuf[1024] ;
	AST_AGI_T	rcvbuf ;

	sprintf( sndbuf, "Action: Login\r\nUsername: %s\r\nSecret: %s\r\n\r\n", "astmgr", "astmgr" ) ;

	if ( sub_send( nsoc, sndbuf, strlen(sndbuf) ) < 0 )
	{
		sub_logput( LOG_ERROR, "send data error: %s", sndbuf ) ;
		return -1 ;
	}

	sub_strip_all_enter( sndbuf ) ;
	sub_logput( LOG_NORML, "SEND:(%s)", sndbuf ) ;

	sub_read_agi(nsoc, &rcvbuf);

	if ( rcvbuf.nstatus != STS_OK )
	{
		sub_logput( LOG_ERROR, "AGI login failed" ) ;
		return STS_NG ;
	}

	return STS_OK ;
}

int sub_agi_event(int nsoc)
{
	char    	sndbuf[1024] ;

    sprintf( sndbuf, "Action: Events\r\nEventmask: On\r\n\r\n");

    if ( sub_send( nsoc, sndbuf, strlen(sndbuf) ) < 0 )
    {
        sub_logput( LOG_ERROR, "send data error: %s", sndbuf ) ;
        return -1 ;
    }

	sub_strip_all_enter( sndbuf ) ;
	sub_logput( LOG_NORML, "SEND:(%s)", sndbuf ) ;

	return STS_OK ;
}

int sub_agi_peers(int nsoc)
{
	char    	sndbuf[1024] ;
	AST_AGI_T	rcvbuf ;

    sprintf( sndbuf, "Action: Sippeers\r\n\r\n");

    if ( sub_send( nsoc, sndbuf, strlen(sndbuf) ) < 0 )
    {
        sub_logput( LOG_ERROR, "send data error: %s", sndbuf ) ;
        return -1 ;
    }

	sub_strip_all_enter( sndbuf ) ;
	sub_logput( LOG_NORML, "SEND:(%s)", sndbuf ) ;

	while ( 1 )
	{
		sub_read_agi(nsoc, &rcvbuf);

		if ( rcvbuf.nstatus != STS_OK )
		{
			break ;
		}
		else if ( rcvbuf.nlistcomplete == TRUE )
		{
			break ;
		}
	}

	return STS_OK ;
}

int sub_read_agi(int nsoc, AST_AGI_T *dstbuf)
{
	char    rcvbuf[1024] ;
	char    wrkbuf[1024] ;
	int		nidx ;

	nidx = 0 ;
	memset( wrkbuf, 0, sizeof(wrkbuf) ) ;
	memset( rcvbuf, 0, sizeof(rcvbuf) ) ;
	memset( dstbuf, 0, sizeof(AST_AGI_T) ) ;

	while ( 1 )
	{
		if ( sub_recv_wait( nsoc, 10 ) < 0 )
		{
			sub_logput( LOG_ERROR, "socket wait failed" ) ;
			break ;
		}

		if ( sub_recv( nsoc, wrkbuf, 1 ) < 0 )
		{
			sub_logput( LOG_ERROR, "socket receive failed" ) ;
			break ;
		}

		if ( wrkbuf[0] == '\n' )
		{
			sub_strip_all_enter( rcvbuf ) ;
			sub_logput( LOG_NORML, "Received: (%s)", rcvbuf ) ;

			if ( strlen( rcvbuf ) == 1 && rcvbuf[0] == ' ' )
			{
				break ;
			}
			
			if ( !strncmp( rcvbuf, AST_STS_ERROR, strlen(AST_STS_ERROR) ) )
			{
				dstbuf->nstatus = STS_NG ;
			}
			else if ( !strncmp( rcvbuf, AST_STS_OK, strlen(AST_STS_OK) ) )
			{
				dstbuf->nstatus = STS_OK ;
			}
			else if ( !strncmp( rcvbuf, AST_EVT_COMP, strlen(AST_EVT_COMP) ) ) 
			{
				dstbuf->nlistcomplete = TRUE ;
			}

			nidx = 0 ;
			memset( wrkbuf, 0, sizeof(wrkbuf) ) ;
    		memset( rcvbuf, 0, sizeof(rcvbuf) ) ;
		}
		else
		{
			rcvbuf[nidx++] = wrkbuf[0] ;
		}
	}
	
	sub_logput( LOG_NORML, "socket read complete" ) ;

	return STS_OK ;
}

